@@ -3918,64 +3918,20 @@ services:
 	assert.Equal(t, build.SBOM, "true")
 }
 
-func TestOverrideMiddle(t *testing.T) {
-	pwd := t.TempDir()
-	base := filepath.Join(pwd, "base.yaml")
-	err := os.WriteFile(base, []byte(`
+func TestNoCacheFilter(t *testing.T) {
+	p, err := loadYAML(`
+name: no-cache-filter
 services:
-  base:
-    volumes:
-      - /foo:/foo
-`), 0o700)
+  string:
+    build:
+      context: .
+      no_cache_filter: foo
+  list:
+    build:
+      context: .
+      no_cache_filter: [foo, bar]
+`)
 	assert.NilError(t, err)
-
-	override := filepath.Join(pwd, "override.yaml")
-	err = os.WriteFile(override, []byte(`
-services:
-  override:
-    extends:
-      file: ./base.yaml
-      service: base
-    volumes: !override
-      -  /bar:/bar
-`), 0o700)
-	assert.NilError(t, err)
-
-	compose := filepath.Join(pwd, "compose.yaml")
-	err = os.WriteFile(compose, []byte(`
-name: test
-services:
-  test:
-    image: test
-    extends:
-      file: ./override.yaml
-      service: override
-    volumes:
-      - /zot:/zot
-`), 0o700)
-	assert.NilError(t, err)
-
-	project, err := LoadWithContext(context.TODO(), types.ConfigDetails{
-		WorkingDir: pwd,
-		ConfigFiles: []types.ConfigFile{
-			{Filename: compose},
-		},
-	})
-	assert.NilError(t, err)
-	test := project.Services["test"]
-	assert.Equal(t, len(test.Volumes), 2)
-	assert.DeepEqual(t, test.Volumes, []types.ServiceVolumeConfig{
-		{
-			Type:   "bind",
-			Source: "/bar",
-			Target: "/bar",
-			Bind:   &types.ServiceVolumeBind{CreateHostPath: true},
-		},
-		{
-			Type:   "bind",
-			Source: "/zot",
-			Target: "/zot",
-			Bind:   &types.ServiceVolumeBind{CreateHostPath: true},
-		},
-	})
+	assert.DeepEqual(t, p.Services["string"].Build.NoCacheFilter, types.StringList{"foo"})
+	assert.DeepEqual(t, p.Services["list"].Build.NoCacheFilter, types.StringList{"foo", "bar"})
 }
