@@ -17,6 +17,7 @@
 package cdi
 
 import (
+	"os"
 	"runtime"
 	"testing"
 
@@ -254,6 +255,14 @@ func TestValidateContainerEdits(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "valid rdt config, root clos",
+			edits: &cdi.ContainerEdits{
+				IntelRdt: &cdi.IntelRdt{
+					ClosID: "/",
+				},
+			},
+		},
 		{
 			name: "invalid rdt config, invalid closID (slash)",
 			edits: &cdi.ContainerEdits{
@@ -290,6 +299,41 @@ func TestValidateContainerEdits(t *testing.T) {
 			},
 			invalid: true,
 		},
+		{
+			name: "valid Linux net device",
+			edits: &cdi.ContainerEdits{
+				NetDevices: []*cdi.LinuxNetDevice{
+					{
+						HostInterfaceName: "eno1",
+						Name:              "netdev0",
+					},
+				},
+			},
+		},
+		{
+			name: "invalid Linux net device, empty host interface name",
+			edits: &cdi.ContainerEdits{
+				NetDevices: []*cdi.LinuxNetDevice{
+					{
+						HostInterfaceName: "",
+						Name:              "netdev0",
+					},
+				},
+			},
+			invalid: true,
+		},
+		{
+			name: "invalid Linux net device, empty container interface name",
+			edits: &cdi.ContainerEdits{
+				NetDevices: []*cdi.LinuxNetDevice{
+					{
+						HostInterfaceName: "eno1",
+						Name:              "",
+					},
+				},
+			},
+			invalid: true,
+		},
 	} {
 		t.Run(tc.name, func(t *testing.T) {
 			edits := ContainerEdits{tc.edits}
@@ -306,6 +350,9 @@ func TestValidateContainerEdits(t *testing.T) {
 func TestApplyContainerEdits(t *testing.T) {
 	nullDeviceMajor := int64(1)
 	nullDeviceMinor := int64(3)
+
+	mode := uint32(0666)
+	nullDeviceFileMode := (*os.FileMode)(&mode)
 	if runtime.GOOS == "darwin" {
 		nullDeviceMajor = 3
 		nullDeviceMinor = 2
@@ -352,10 +399,11 @@ func TestApplyContainerEdits(t *testing.T) {
 				Linux: &oci.Linux{
 					Devices: []oci.LinuxDevice{
 						{
-							Path:  "/dev/null",
-							Type:  "c",
-							Major: nullDeviceMajor,
-							Minor: nullDeviceMinor,
+							Path:     "/dev/null",
+							Type:     "c",
+							Major:    nullDeviceMajor,
+							Minor:    nullDeviceMinor,
+							FileMode: nullDeviceFileMode,
 						},
 					},
 					Resources: &oci.LinuxResources{
@@ -395,10 +443,11 @@ func TestApplyContainerEdits(t *testing.T) {
 				Linux: &oci.Linux{
 					Devices: []oci.LinuxDevice{
 						{
-							Path:  "/dev/null",
-							Type:  "c",
-							Major: nullDeviceMajor,
-							Minor: nullDeviceMinor,
+							Path:     "/dev/null",
+							Type:     "c",
+							Major:    nullDeviceMajor,
+							Minor:    nullDeviceMinor,
+							FileMode: nullDeviceFileMode,
 						},
 					},
 					Resources: &oci.LinuxResources{
@@ -524,21 +573,21 @@ func TestApplyContainerEdits(t *testing.T) {
 			spec: &oci.Spec{},
 			edits: &cdi.ContainerEdits{
 				IntelRdt: &cdi.IntelRdt{
-					ClosID:        "clos-1",
-					L3CacheSchema: "L3:0=ff;1=ff",
-					MemBwSchema:   "MB:0=50;1=50",
-					EnableCMT:     true,
-					EnableMBM:     true,
+					ClosID:           "clos-1",
+					L3CacheSchema:    "L3:0=ff;1=ff",
+					MemBwSchema:      "MB:0=50;1=50",
+					Schemata:         []string{"L2:0=ffff"},
+					EnableMonitoring: true,
 				},
 			},
 			result: &oci.Spec{
 				Linux: &oci.Linux{
 					IntelRdt: &oci.LinuxIntelRdt{
-						ClosID:        "clos-1",
-						L3CacheSchema: "L3:0=ff;1=ff",
-						MemBwSchema:   "MB:0=50;1=50",
-						EnableCMT:     true,
-						EnableMBM:     true,
+						ClosID:           "clos-1",
+						L3CacheSchema:    "L3:0=ff;1=ff",
+						MemBwSchema:      "MB:0=50;1=50",
+						Schemata:         []string{"L2:0=ffff"},
+						EnableMonitoring: true,
 					},
 				},
 			},
@@ -548,25 +597,91 @@ func TestApplyContainerEdits(t *testing.T) {
 			spec: &oci.Spec{
 				Linux: &oci.Linux{
 					IntelRdt: &oci.LinuxIntelRdt{
-						ClosID:        "clos-1",
-						L3CacheSchema: "L3:0=ff",
-						MemBwSchema:   "MB:0=100",
-						EnableCMT:     true,
-						EnableMBM:     true,
+						ClosID:           "clos-1",
+						L3CacheSchema:    "L3:0=ff",
+						MemBwSchema:      "MB:0=100",
+						Schemata:         []string{"L2:0=ffff"},
+						EnableMonitoring: true,
 					},
 				},
 			},
 			edits: &cdi.ContainerEdits{
 				IntelRdt: &cdi.IntelRdt{
-					ClosID:        "clos-2",
-					L3CacheSchema: "L3:0=f",
+					ClosID:           "clos-2",
+					L3CacheSchema:    "L3:0=f",
+					EnableMonitoring: false,
 				},
 			},
 			result: &oci.Spec{
 				Linux: &oci.Linux{
 					IntelRdt: &oci.LinuxIntelRdt{
-						ClosID:        "clos-2",
-						L3CacheSchema: "L3:0=f",
+						ClosID:           "clos-2",
+						L3CacheSchema:    "L3:0=f",
+						EnableMonitoring: false,
+					},
+				},
+			},
+		},
+		{
+			name: "empty spec, Linux net devices",
+			spec: &oci.Spec{},
+			edits: &cdi.ContainerEdits{
+				NetDevices: []*cdi.LinuxNetDevice{
+					{
+						HostInterfaceName: "eno1",
+						Name:              "netdev0",
+					},
+					{
+						HostInterfaceName: "eno2",
+						Name:              "netdev1",
+					},
+				},
+			},
+			result: &oci.Spec{
+				Linux: &oci.Linux{
+					NetDevices: map[string]oci.LinuxNetDevice{
+						"eno1": {
+							Name: "netdev0",
+						},
+						"eno2": {
+							Name: "netdev1",
+						},
+					},
+				},
+			},
+		},
+		{
+			name: "non-empty spec, overriding Linux net devices",
+			spec: &oci.Spec{
+				Linux: &oci.Linux{
+					NetDevices: map[string]oci.LinuxNetDevice{
+						"eno1": {
+							Name: "netdev1",
+						},
+					},
+				},
+			},
+			edits: &cdi.ContainerEdits{
+				NetDevices: []*cdi.LinuxNetDevice{
+					{
+						HostInterfaceName: "eno1",
+						Name:              "netdev2",
+					},
+					{
+						HostInterfaceName: "eno2",
+						Name:              "netdev1",
+					},
+				},
+			},
+			result: &oci.Spec{
+				Linux: &oci.Linux{
+					NetDevices: map[string]oci.LinuxNetDevice{
+						"eno1": {
+							Name: "netdev2",
+						},
+						"eno2": {
+							Name: "netdev1",
+						},
 					},
 				},
 			},
@@ -776,6 +891,7 @@ func TestAppend(t *testing.T) {
 						IntelRdt: &cdi.IntelRdt{
 							ClosID:        "clos-1",
 							L3CacheSchema: "L3:0=ff",
+							Schemata:      []string{"L2: 0xFFFF"},
 						},
 					},
 				},
